"""
 Copyright (c) 2023 Anthony Mugendi
 
 This software is released under the MIT License.
 https://opensource.org/licenses/MIT
"""

from .lists import listify
from .classes import is_initialized, is_class

# pick values from list/dict/tuple/class or set
def pick(obj, keys, sort_keys=True):
    if not isinstance(obj, (dict, list, tuple, set)) and not is_initialized(obj):
        raise KeyError(f"obj must be a dict, list, tuple, set or an initialized class object")
    if not isinstance(keys, (tuple, set, list, str, int)):
        raise KeyError(f"keys to pick must be a string, integer, set, tuple or list")

    keys = listify(keys)

    if sort_keys:
        keys = sorted(keys)

    if isinstance(obj, dict):
        return {k: obj[k] for k in keys if k in obj}

    elif isinstance(obj, tuple):
        l = len(obj)
        return tuple([obj[k] for k in keys if k <= l - 1])

    elif isinstance(obj, set):
        return {k for k in keys if k in obj}

    elif is_class(obj):
        return {k: getattr(obj, k) for k in keys if hasattr(obj, str(k))}

    else:
        l = len(obj)
        return [obj[k] for k in keys if k <= l - 1]



"""
 Copyright (c) 2023 Anthony Mugendi
 
 This software is released under the MIT License.
 https://opensource.org/licenses/MIT
"""
import types 
import re


# is variable a class?
def is_class(cls):
    return str(type(cls)).startswith("<class") and hasattr(cls, "__weakref__")

# check if a class is initialized
def is_initialized(cls):
    new_cls = types.new_class("_")
    return is_class(cls) and type(cls) != type(new_cls)


# get properties of this class
def get_props(cls, filter_callable=False, with_meta=False):
    default_props = dir(types.new_class("_"))

    resp = {} if with_meta else set()
    
    for p in dir(cls):
        if p not in default_props and (
            getattr(cls, p) if not filter_callable else callable(getattr(cls, p))
        ):
            if with_meta:
                resp[p] = {
                    "type": re.match(r"<class\s+'([^>']+)", str(type(getattr(cls, p))))[
                        1
                    ],
                    "value" : getattr(cls, p) if not callable(getattr(cls, p)) else 'N/A'
                }

            else:
                resp.add(p)

    return resp



"""
 Copyright (c) 2023 Anthony Mugendi
 
 This software is released under the MIT License.
 https://opensource.org/licenses/MIT
"""

def listify(v):
    if isinstance(v, (tuple, set)):
        v = list(v)
    else:
        v = v if isinstance(v, list) else [v]

    return v